package ca.recrown.islandsurvivalcraft.world;

import java.util.Random;

import org.bukkit.World;

import ca.recrown.islandsurvivalcraft.world.generation.chunks.IslandWorldChunkGenerator;

public class WorldInfo {
    private final WorldInfoManager manager;
    private volatile boolean initialized;
    public volatile long seed;
    public volatile BiomeMap biomeMap;
    public volatile TemperatureMap tempMap;
    public volatile IslandWorldMap islandMap;
    public volatile IslandWorldChunkGenerator generator;

    /**
     * Will initialize with the known information extracted from the given world if world is not null.
     * If the world is null, this world info will not be initialized and awaits initialization from the generator.
     * @param world The world this object is describing. May be null to create a non-initialized info object.
     */
    public WorldInfo(WorldInfoManager manager, World world) {
        this.manager = manager;
        if (world != null) {
            initialize(world);
        }
    }

    public void initialize(World world, IslandWorldChunkGenerator chunkGenerator) {
        if (initialized) throw new IllegalStateException("This world information object has already been initialized.");
        initialized = true;
        this.seed = world.getSeed();
        Random random = new Random(seed);
        biomeMap = new BiomeMap(random);
        tempMap = new TemperatureMap(random);
        islandMap = new IslandWorldMap(random);
        generator = chunkGenerator;
        generator.initialize(random, true, world.getSeaLevel(), world.getMaxHeight());
        manager.register(world, this);
    }

    private void initialize(World world) {
        if (initialized) throw new IllegalStateException("This world information object has already been initialized.");
        initialized = true;
        this.seed = world.getSeed();
        Random random = new Random(seed);
        biomeMap = new BiomeMap(random);
        tempMap = new TemperatureMap(random);
        islandMap = new IslandWorldMap(random);
        generator = new IslandWorldChunkGenerator(this);
        generator.initialize(random, true, world.getSeaLevel(), world.getMaxHeight());
    }

    /**
     * @return whether or not this world info is initialized.
     */
    public boolean isInitialized() {
        return initialized;
    }
}